import tkinter as tk
import tkinter.messagebox
import customtkinter
from tkinter.filedialog import askopenfilename
from tkinter.filedialog import asksaveasfilename
from tkinter import simpledialog
import os
from .GUI_postprocessing import *
from .GUI_POSEA import POSEA, POSEA2, POSEA3
from PIL import Image

customtkinter.set_appearance_mode("Light")  # Modes: "Dark", "Light"
customtkinter.set_default_color_theme("dark-blue")  # Themes: "blue" (standard), "green", "dark-blue"


class CustomInputDialog(tk.simpledialog.Dialog):
    def __init__(self, parent, title=None, prompt=None):
        self.prompt = prompt
        super().__init__(parent, title)

    def body(self, master):
        self.prompt_label = tk.Label(master, text=self.prompt)
        self.prompt_label.pack()

        self.entry = tk.Entry(master, width=40)
        self.entry.pack()

    def apply(self):
        self.result = self.entry.get()

class App(customtkinter.CTk):

    def __init__(self):
        super().__init__()
        self.k_value_input = 1.8

        # configure window
        self.title("PostProcessing & Evaluation")
        self.geometry(f"{1300}x{900}")

        # configure grid layout (4x4)
        self.grid_columnconfigure(1, weight=1)
        self.grid_columnconfigure((2, 3), weight=0)
        self.grid_rowconfigure((0, 1, 2), weight=1)

        # create menu
        self.menubar = tk.Menu(self)
        self.filemenu = tk.Menu(self.menubar, tearoff=0)
        self.filemenu.add_command(label="Load original image (*.tif,*.png,*.jpg)", command=self.open_file1)
        self.filemenu.add_command(label="Load segmentation result (*.tif,*.png,*.jpg)", command=self.open_file2)
        self.filemenu.add_command(label="Load reference image (cell mask) (*.tif,*.png,*.jpg)", command=self.open_file3)
        self.filemenu.add_command(label="Load reference image (cytoplasm mask) (*.tif,*.png,*.jpg)", command=self.open_file4)
        self.filemenu.add_command(label="Load reference image (nucleus mask) (*.tif,*.png,*.jpg)", command=self.open_file5)
        self.filemenu.add_command(label="Save cytoplasm mask as tif", command=self.saveimage1)
        self.filemenu.add_command(label="Save nucleus mask as tif", command=self.saveimage2)
        self.filemenu.add_command(label="Save per cell accuracy (*.xlsx)", command=self.savedata)
        self.menubar.add_cascade(menu=self.filemenu, label="File")

        onlyone = tk.BooleanVar()

        self.modelmenu = tk.Menu(self.menubar, tearoff=0)
        self.modelmenu.add_checkbutton(label="isodata", onvalue=1, offvalue=0, variable=onlyone, command=self.isodata_process)
        self.modelmenu.add_checkbutton(label="li", onvalue=2, offvalue=0, variable=onlyone, command=self.li_process)
        self.modelmenu.add_checkbutton(label="mean", onvalue=3, offvalue=0, variable=onlyone, command=self.mean_process)
        self.modelmenu.add_checkbutton(label="otsu", onvalue=4, offvalue=0, variable=onlyone, command=self.otsu_process)
        self.modelmenu.add_checkbutton(label="triangle", onvalue=5, offvalue=0, variable=onlyone, command=self.triangle_process)
        self.modelmenu.add_checkbutton(label="yen", onvalue=6, offvalue=0, variable=onlyone, command=self.yen_process)
        self.modelmenu.add_separator()
        self.modelmenu.add_command(label="find best (cytoplasm)", command=self.findbest_cyto)
        self.modelmenu.add_command(label="find best (nucleus)", command=self.findbest_nucl)
        self.menubar.add_cascade(menu=self.modelmenu, label="Models")

        onlytwo = tk.BooleanVar()

        self.calculatemenu = tk.Menu(self.menubar, tearoff=0)
        self.calculatemenu.add_checkbutton(label="Set a number of k", onvalue=1, offvalue=0, variable=onlytwo, command=self.k_value)
        self.menubar.add_cascade(menu=self.calculatemenu, label="k values")

        onlythree = tk.BooleanVar()

        self.calculatemenu = tk.Menu(self.menubar, tearoff=0)
        self.calculatemenu.add_checkbutton(label="Get cytoplasm & nucleus mask", onvalue=1, offvalue=0, variable=onlythree, command=self.showresult)
        self.menubar.add_cascade(menu=self.calculatemenu, label="Post-processing")

        onlyfour = tk.BooleanVar()

        self.evaluatemenu = tk.Menu(self.menubar, tearoff=0)
        self.evaluatemenu.add_checkbutton(label="POSEA", onvalue=1, offvalue=0, variable=onlyfour, command=self.POSEA_process1)
        self.evaluatemenu.add_separator()
        self.evaluatemenu.add_checkbutton(label="POSEA (cyto)", onvalue=2, offvalue=0, variable=onlyfour, command=self.POSEA_process2)
        self.evaluatemenu.add_checkbutton(label="POSEA (nucl)", onvalue=3, offvalue=0, variable=onlyfour, command=self.POSEA_process3)
        self.menubar.add_cascade(menu=self.evaluatemenu, label="Evaluation")

        self.helpmenu = tk.Menu(self.menubar, tearoff=0)
        self.helpmenu.add_command(label="Help with GUI", command=self.show_message)
        self.menubar.add_cascade(menu=self.helpmenu, label="Help")
        self.config(menu=self.menubar)



        # create first frame
        self.first_frame = customtkinter.CTkFrame(self, corner_radius=0, fg_color="transparent")
        self.first_frame.grid_columnconfigure(0, weight=1)


        # create second frame
        self.second_frame = customtkinter.CTkFrame(self, corner_radius=0, fg_color="transparent")
        self.second_frame.grid_columnconfigure(0, weight=1)


        # create third frame
        self.third_frame = customtkinter.CTkFrame(self, corner_radius=0, fg_color="transparent")
        self.third_frame.grid_columnconfigure(0, weight=1)

        # create fourth frame
        self.fourth_frame = customtkinter.CTkFrame(self, corner_radius=0, fg_color="transparent")
        self.fourth_frame.grid_columnconfigure(0, weight=1)

        # create fifth frame
        self.fifth_frame = customtkinter.CTkFrame(self, corner_radius=0, fg_color="transparent")
        self.fifth_frame.grid_columnconfigure(0, weight=1)

        # create sixth frame
        self.sixth_frame = customtkinter.CTkFrame(self, corner_radius=0, fg_color="transparent")
        self.sixth_frame.grid_columnconfigure(0, weight=1)

        # create seventh frame
        self.seventh_frame = customtkinter.CTkFrame(self, corner_radius=0, fg_color="transparent")
        self.seventh_frame.grid_columnconfigure(0, weight=1)


        # create sidebar frame with widgets
        self.sidebar_frame = customtkinter.CTkFrame(self, width=140, corner_radius=0)
        self.sidebar_frame.grid(row=0, column=0, rowspan=7, sticky="nsew")
        self.sidebar_frame.grid_rowconfigure(8, weight=1)
        self.logo_label = customtkinter.CTkLabel(self.sidebar_frame, text="Walsh Lab", font=customtkinter.CTkFont(size=20, weight="bold"))
        self.logo_label.grid(row=0, column=0, padx=20, pady=(20, 10))
        self.sidebar_button_1 = customtkinter.CTkButton(self.sidebar_frame, text="Original Image", fg_color="transparent", text_color=("gray10", "gray90"),
                                                        hover_color=("gray70", "gray30"),command=self.sidebar_button_1_event)
        self.sidebar_button_1.grid(row=1, column=0, padx=20, pady=10)
        self.sidebar_button_2 = customtkinter.CTkButton(self.sidebar_frame, text="Segmentation Mask", fg_color="transparent", text_color=("gray10", "gray90"),
                                                        hover_color=("gray70", "gray30"),command=self.sidebar_button_2_event)
        self.sidebar_button_2.grid(row=2, column=0, padx=20, pady=10)
        self.sidebar_button_3 = customtkinter.CTkButton(self.sidebar_frame, text="Reference Image (cell mask)", fg_color="transparent", text_color=("gray10", "gray90"),
                                                        hover_color=("gray70", "gray30"),command=self.sidebar_button_3_event)
        self.sidebar_button_3.grid(row=3, column=0, padx=20, pady=10)
        self.sidebar_button_4 = customtkinter.CTkButton(self.sidebar_frame, text="Reference Image (cytoplasm mask)", fg_color="transparent", text_color=("gray10", "gray90"),
                                                        hover_color=("gray70", "gray30"),command=self.sidebar_button_4_event)
        self.sidebar_button_4.grid(row=4, column=0, padx=20, pady=10)
        self.sidebar_button_5 = customtkinter.CTkButton(self.sidebar_frame, text="Reference Image (nucleus mask)", fg_color="transparent", text_color=("gray10", "gray90"),
                                                        hover_color=("gray70", "gray30"),command=self.sidebar_button_5_event)
        self.sidebar_button_5.grid(row=5, column=0, padx=20, pady=10)
        self.sidebar_button_6 = customtkinter.CTkButton(self.sidebar_frame, text="Cytoplasm Mask", font=customtkinter.CTkFont(size=15, weight="bold"), fg_color="transparent", text_color=("gray10", "gray90"),
                                                        hover_color=("gray70", "gray30"),command=self.sidebar_button_6_event)
        self.sidebar_button_6.grid(row=6, column=0, padx=20, pady=10)
        self.sidebar_button_7 = customtkinter.CTkButton(self.sidebar_frame, text="Nucleus Mask", font=customtkinter.CTkFont(size=15, weight="bold"), fg_color="transparent", text_color=("gray10", "gray90"),
                                                        hover_color=("gray70", "gray30"),command=self.sidebar_button_7_event)
        self.sidebar_button_7.grid(row=7, column=0, padx=20, pady=10)


        self.appearance_mode_label = customtkinter.CTkLabel(self.sidebar_frame, text="Appearance Mode:", anchor="w")
        self.appearance_mode_label.grid(row=9, column=0, padx=20, pady=(10, 0))
        self.appearance_mode_optionemenu = customtkinter.CTkOptionMenu(self.sidebar_frame, values=["Light", "Dark"],
                                                                       command=self.change_appearance_mode_event)
        self.appearance_mode_optionemenu.grid(row=10, column=0, padx=20, pady=(10, 10))
        self.scaling_label = customtkinter.CTkLabel(self.sidebar_frame, text="UI Scaling:", anchor="w")
        self.scaling_label.grid(row=11, column=0, padx=20, pady=(10, 0))
        self.scaling_optionemenu = customtkinter.CTkOptionMenu(self.sidebar_frame, values=["80%", "90%", "100%", "110%", "120%"],
                                                               command=self.change_scaling_event)
        self.scaling_optionemenu.grid(row=12, column=0, padx=20, pady=(10, 20))

        # create evaluation result frame
        self.evaluation_res_frame = customtkinter.CTkFrame(self)
        self.evaluation_res_frame.grid(row=0, column=3, padx=(20, 20), pady=(20, 0), sticky="nsew")
        self.label_radio_group = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Evaluation Results", font=customtkinter.CTkFont(size=18, weight="bold"))
        self.label_radio_group.grid(row=0, column=2, columnspan=1, padx=10, pady=10, sticky="")
        self.evaluation_res_1 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Traditional Evaluation", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_1.grid(row=1, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=2, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=3, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=4, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_2 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (entire image)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_2.grid(row=5, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=6, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=7, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=8, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_3 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (per cell)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_3.grid(row=9, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=10, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=11, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall:", font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=12, column=2, pady=10, padx=20, sticky="w")

        # kind reminder
        self.protocol("WM_DELETE_WINDOW", self.on_closing)

        # set default values
        self.appearance_mode_optionemenu.set("Light")
        self.scaling_optionemenu.set("100%")


    def change_appearance_mode_event(self, new_appearance_mode: str):
        customtkinter.set_appearance_mode(new_appearance_mode)

    def change_scaling_event(self, new_scaling: str):
        new_scaling_float = int(new_scaling.replace("%", "")) / 100
        customtkinter.set_widget_scaling(new_scaling_float)

    def on_closing(self):
        if tkinter.messagebox.askyesno(title="Quit?", message="Do you really want to quit?"):
            self.destroy()


    def select_frame_by_name(self, name):
        # set button color for selected button
        self.sidebar_button_1.configure(fg_color=("gray75", "gray25") if name == "Original Image" else "transparent")
        self.sidebar_button_2.configure(fg_color=("gray75", "gray25") if name == "Segmentation Mask" else "transparent")
        self.sidebar_button_3.configure(fg_color=("gray75", "gray25") if name == "Reference Image (cell mask)" else "transparent")
        self.sidebar_button_4.configure(fg_color=("gray75", "gray25") if name == "Reference Image (cytoplasm mask)" else "transparent")
        self.sidebar_button_5.configure(fg_color=("gray75", "gray25") if name == "Reference Image (nucleus mask)" else "transparent")
        self.sidebar_button_6.configure(fg_color=("gray75", "gray25") if name == "Cytoplasm Mask" else "transparent")
        self.sidebar_button_7.configure(fg_color=("gray75", "gray25") if name == "Nucleus Mask" else "transparent")


        # show selected frame
        if name == "Original Image":
            self.first_frame.grid(row=0, column=1, sticky="nsew")
        else:
            self.first_frame.grid_forget()
        if name == "Segmentation Mask":
            self.second_frame.grid(row=0, column=1, sticky="nsew")
        else:
            self.second_frame.grid_forget()
        if name == "Reference Image (cell mask)":
            self.third_frame.grid(row=0, column=1, sticky="nsew")
        else:
            self.third_frame.grid_forget()
        if name == "Reference Image (cytoplasm mask)":
            self.fourth_frame.grid(row=0, column=1, sticky="nsew")
        else:
            self.fourth_frame.grid_forget()
        if name == "Reference Image (nucleus mask)":
            self.fifth_frame.grid(row=0, column=1, sticky="nsew")
        else:
            self.fifth_frame.grid_forget()
        if name == "Cytoplasm Mask":
            self.sixth_frame.grid(row=0, column=1, sticky="nsew")
        else:
            self.sixth_frame.grid_forget()
        if name == "Nucleus Mask":
            self.seventh_frame.grid(row=0, column=1, sticky="nsew")
        else:
            self.seventh_frame.grid_forget()

    def sidebar_button_1_event(self):
        self.select_frame_by_name("Original Image")

    def sidebar_button_2_event(self):
        self.select_frame_by_name("Segmentation Mask")

    def sidebar_button_3_event(self):
        self.select_frame_by_name("Reference Image (cell mask)")

    def sidebar_button_4_event(self):
        self.select_frame_by_name("Reference Image (cytoplasm mask)")

    def sidebar_button_5_event(self):
        self.select_frame_by_name("Reference Image (nucleus mask)")

    def sidebar_button_6_event(self):
        self.select_frame_by_name("Cytoplasm Mask")

    def sidebar_button_7_event(self):
        self.select_frame_by_name("Nucleus Mask")


    def show_message(self):
        tkinter.messagebox.showinfo(title="Help", message="contact nwang27@tamu.edu")


    def open_file1(self):
        curr_directory = os.getcwd()
        self.file1 = askopenfilename(initialdir=curr_directory, title="Select Image",
                                    filetypes=[('tif file', '*.tif'), ('tiff file', '*.tiff'), ('png file', '*.png'), ('jpg file', '*.jpg'), ('all files', '*.')])
        if self.file1 is not None:
            self.img1 = customtkinter.CTkImage(Image.open(self.file1), size=(800, 800))
            self.first_frame_image_label = customtkinter.CTkLabel(self.first_frame, text="", image=self.img1)
            self.first_frame_image_label.grid(row=0, column=1, padx=25, pady=40)
        else:
            tkinter.messagebox.showinfo(title="Error", message="Please upload the original image")


    def open_file2(self):
        curr_directory = os.getcwd()
        self.file2 = askopenfilename(initialdir=curr_directory, title="Select Image",
                                    filetypes=[('tif file', '*.tif'), ('tiff file', '*.tiff'), ('png file', '*.png'), ('jpg file', '*.jpg'), ('all files', '*.')])
        if self.file2 is not None:
            self.img2 = customtkinter.CTkImage(Image.open(self.file2), size=(800, 800))
            self.second_frame_image_label = customtkinter.CTkLabel(self.second_frame, text="", image=self.img2)
            self.second_frame_image_label.grid(row=0, column=1, padx=25, pady=40)
        else:
            tkinter.messagebox.showinfo(title="Error", message="Please upload the segmentation mask")


    def open_file3(self):
        curr_directory = os.getcwd()
        self.file3 = askopenfilename(initialdir=curr_directory, title="Select Image",
                                    filetypes=[('tif file', '*.tif'), ('tiff file', '*.tiff'), ('png file', '*.png'), ('jpg file', '*.jpg'), ('all files', '*.')])
        if self.file3 is not None:
            self.img3 = customtkinter.CTkImage(Image.open(self.file3), size=(800, 800))
            self.third_frame_image_label = customtkinter.CTkLabel(self.third_frame, text="", image=self.img3)
            self.third_frame_image_label.grid(row=0, column=1, padx=25, pady=40)
        else:
            tkinter.messagebox.showinfo(title="Error", message="Please upload the reference image (cell mask)")


    def open_file4(self):
        curr_directory = os.getcwd()
        self.file4 = askopenfilename(initialdir=curr_directory, title="Select Image",
                                    filetypes=[('tif file', '*.tif'), ('tiff file', '*.tiff'), ('png file', '*.png'), ('jpg file', '*.jpg'), ('all files', '*.')])
        if self.file4 is not None:
            self.img4 = customtkinter.CTkImage(Image.open(self.file4), size=(800, 800))
            self.fourth_frame_image_label = customtkinter.CTkLabel(self.fourth_frame, text="", image=self.img4)
            self.fourth_frame_image_label.grid(row=0, column=1, padx=25, pady=40)
        else:
            tkinter.messagebox.showinfo(title="Error", message="Please upload the reference image (cytoplasm mask)")


    def open_file5(self):
        curr_directory = os.getcwd()
        self.file5 = askopenfilename(initialdir=curr_directory, title="Select Image",
                                    filetypes=[('tif file', '*.tif'), ('tiff file', '*.tiff'), ('png file', '*.png'), ('jpg file', '*.jpg'), ('all files', '*.')])
        if self.file5 is not None:
            self.img5 = customtkinter.CTkImage(Image.open(self.file5), size=(800, 800))
            self.fifth_frame_image_label = customtkinter.CTkLabel(self.fifth_frame, text="", image=self.img5)
            self.fifth_frame_image_label.grid(row=0, column=1, padx=25, pady=40)
        else:
            tkinter.messagebox.showinfo(title="Error", message="Please upload the reference image (nucleus mask)")


    def saveimage1(self):
        curr_directory = os.getcwd()
        self.file_save1 = asksaveasfilename(initialdir=curr_directory, defaultextension='.tif', title="Save Image")
        if self.file_save1 is not None:
            self.result_cyto.save(self.file_save1)
        else:
            tkinter.messagebox.showinfo(title="Error", message="There is no result found")

    def saveimage2(self):
        curr_directory = os.getcwd()
        self.file_save2 = asksaveasfilename(initialdir=curr_directory, defaultextension='.tif', title="Save Image")
        if self.file_save2 is not None:
            self.result_nucl.save(self.file_save2)
        else:
            tkinter.messagebox.showinfo(title="Error", message="There is no result found")

    def savedata(self):
        curr_directory = os.getcwd()
        self.file_save3 = asksaveasfilename(initialdir=curr_directory, defaultextension='.xlsx', title="Save Data")
        if self.file_save3 is not None:
            self.data.to_excel(self.file_save3)
        else:
            tkinter.messagebox.showinfo(title="Error", message="There is no result found")


    #########image processing
    def k_value(self):
        dialog = CustomInputDialog(self, title="k Value", prompt=f"Enter the value of k (Current value: {self.k_value_input}):")
        self.k_value_input = float(dialog.result)
        if self.k_value_input:
            print("k value:", self.k_value_input)
        else:
            tk.messagebox.showinfo(title="Error", message="No value entered.")


    def isodata_process(self):
        self.img_cyto, self.img_nucl = isodata(self.file1, self.file2, self.k_value_input)

    def li_process(self):
        self.img_cyto, self.img_nucl = li(self.file1, self.file2, self.k_value_input)

    def mean_process(self):
        self.img_cyto, self.img_nucl = mean(self.file1, self.file2, self.k_value_input)

    def otsu_process(self):
        self.img_cyto, self.img_nucl = otsu(self.file1, self.file2, self.k_value_input)

    def triangle_process(self):
        self.img_cyto, self.img_nucl = triangle(self.file1, self.file2, self.k_value_input)

    def yen_process(self):
        self.img_cyto, self.img_nucl = yen(self.file1, self.file2, self.k_value_input)

    def POSEA_process1(self):
        self.F_mask1, self.Pre_mask1, self.Re_mask1, self.F_obj1, self.Pre_obj1, \
        self.Re_obj1, self.cell_each_F1, self.cell_each_P1, self.cell_each_R1, self.data = POSEA(self.file2, self.file3)
        self.evaluation_res_frame = customtkinter.CTkFrame(self)
        self.evaluation_res_frame.grid(row=0, column=3, padx=(20, 20), pady=(20, 0), sticky="nsew")
        self.label_radio_group = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Evaluation Results", font=customtkinter.CTkFont(size=18, weight="bold"))
        self.label_radio_group.grid(row=0, column=2, columnspan=1, padx=10, pady=10, sticky="")
        self.evaluation_res_1 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Traditional Evaluation", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_1.grid(row=1, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.F_mask1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=2, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.Pre_mask1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=3, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.Re_mask1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=4, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_2 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (entire image)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_2.grid(row=5, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.F_obj1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=6, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.Pre_obj1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=7, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.Re_obj1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=8, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_3 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (per cell)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_3.grid(row=9, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.cell_each_F1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=10, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.cell_each_P1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=11, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.cell_each_R1), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=12, column=2, pady=10, padx=20, sticky="w")

    def POSEA_process2(self):
        self.F_mask2, self.Pre_mask2, self.Re_mask2, self.F_obj2, self.Pre_obj2, \
        self.Re_obj2, self.cell_each_F2, self.cell_each_P2, self.cell_each_R2, self.data = POSEA2(self.result_cyto, self.file3, self.file4)
        self.evaluation_res_frame = customtkinter.CTkFrame(self)
        self.evaluation_res_frame.grid(row=0, column=3, padx=(20, 20), pady=(20, 0), sticky="nsew")
        self.label_radio_group = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Evaluation Results", font=customtkinter.CTkFont(size=18, weight="bold"))
        self.label_radio_group.grid(row=0, column=2, columnspan=1, padx=10, pady=10, sticky="")
        self.evaluation_res_1 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Traditional Evaluation", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_1.grid(row=1, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.F_mask2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=2, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.Pre_mask2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=3, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.Re_mask2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=4, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_2 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (entire image)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_2.grid(row=5, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.F_obj2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=6, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.Pre_obj2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=7, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.Re_obj2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=8, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_3 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (per cell)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_3.grid(row=9, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.cell_each_F2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=10, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.cell_each_P2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=11, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.cell_each_R2), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=12, column=2, pady=10, padx=20, sticky="w")

    def POSEA_process3(self):
        self.F_mask3, self.Pre_mask3, self.Re_mask3, self.F_obj3, self.Pre_obj3, \
        self.Re_obj3, self.cell_each_F3, self.cell_each_P3, self.cell_each_R3, self.data = POSEA2(self.result_nucl, self.file3, self.file5)
        self.evaluation_res_frame = customtkinter.CTkFrame(self)
        self.evaluation_res_frame.grid(row=0, column=3, padx=(20, 20), pady=(20, 0), sticky="nsew")
        self.label_radio_group = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Evaluation Results", font=customtkinter.CTkFont(size=18, weight="bold"))
        self.label_radio_group.grid(row=0, column=2, columnspan=1, padx=10, pady=10, sticky="")
        self.evaluation_res_1 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Traditional Evaluation", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_1.grid(row=1, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.F_mask3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=2, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.Pre_mask3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=3, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.Re_mask3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=4, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_2 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (entire image)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_2.grid(row=5, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.F_obj3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=6, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.Pre_obj3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=7, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.Re_obj3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=8, column=2, pady=10, padx=20, sticky="w")

        self.evaluation_res_3 = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="POSEA (per cell)", font=customtkinter.CTkFont(size=16))
        self.evaluation_res_3.grid(row=9, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_F = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="F-measure: " + str(self.cell_each_F3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_F.grid(row=10, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_P = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Precision: " + str(self.cell_each_P3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_P.grid(row=11, column=2, pady=10, padx=20, sticky="w")
        self.evaluation_res_R = customtkinter.CTkLabel(master=self.evaluation_res_frame, text="Recall: " + str(self.cell_each_R3), font=customtkinter.CTkFont(size=14))
        self.evaluation_res_R.grid(row=12, column=2, pady=10, padx=20, sticky="w")


    def findbest_cyto(self):
        self.img_cyto1, _= isodata(self.file1, self.file2, self.k_value_input)
        self.img_cyto2, _ = li(self.file1, self.file2, self.k_value_input)
        self.img_cyto3, _ = mean(self.file1, self.file2, self.k_value_input)
        self.img_cyto4, _ = otsu(self.file1, self.file2, self.k_value_input)
        self.img_cyto5, _ = triangle(self.file1, self.file2, self.k_value_input)
        self.img_cyto6, _ = yen(self.file1, self.file2, self.k_value_input)

        self.F_isodata1 = POSEA3(self.img_cyto1, self.file3, self.file4)
        self.F_li1 = POSEA3(self.img_cyto2, self.file3, self.file4)
        self.F_mean1 = POSEA3(self.img_cyto3, self.file3, self.file4)
        self.F_otsu1 = POSEA3(self.img_cyto4, self.file3, self.file4)
        self.F_triangle1 = POSEA3(self.img_cyto5, self.file3, self.file4)
        self.F_yen1 = POSEA3(self.img_cyto6, self.file3, self.file4)

        F_list1 = [self.F_isodata1, self.F_li1, self.F_mean1, self.F_otsu1, self.F_triangle1, self.F_yen1]

        if self.F_isodata1 == max(F_list1):
            tkinter.messagebox.showinfo(title="Find best (cytoplasm)", message="Choose isodata")
        elif self.F_li1 == max(F_list1):
            tkinter.messagebox.showinfo(title="Find best (cytoplasm)", message="Choose li")
        elif self.F_mean1 == max(F_list1):
            tkinter.messagebox.showinfo(title="Find best (cytoplasm)", message="Choose mean")
        elif self.F_otsu1 == max(F_list1):
            tkinter.messagebox.showinfo(title="Find best (cytoplasm)", message="Choose otsu")
        elif self.F_triangle1 == max(F_list1):
            tkinter.messagebox.showinfo(title="Find best (cytoplasm)", message="Choose triangle")
        else:
            tkinter.messagebox.showinfo(title="Find best (cytoplasm)", message="Choose yen")

    def findbest_nucl(self):
        _, self.img_nucl1 = isodata(self.file1, self.file2, self.k_value_input)
        _, self.img_nucl2 = li(self.file1, self.file2, self.k_value_input)
        _, self.img_nucl3 = mean(self.file1, self.file2, self.k_value_input)
        _, self.img_nucl4 = otsu(self.file1, self.file2, self.k_value_input)
        _, self.img_nucl5 = triangle(self.file1, self.file2, self.k_value_input)
        _, self.img_nucl6 = yen(self.file1, self.file2, self.k_value_input)

        self.F_isodata2 = POSEA3(self.img_nucl1, self.file3, self.file5)
        self.F_li2 = POSEA3(self.img_nucl2, self.file3, self.file5)
        self.F_mean2 = POSEA3(self.img_nucl3, self.file3, self.file5)
        self.F_otsu2 = POSEA3(self.img_nucl4, self.file3, self.file5)
        self.F_triangle2 = POSEA3(self.img_nucl5, self.file3, self.file5)
        self.F_yen2 = POSEA3(self.img_nucl6, self.file3, self.file5)


        F_list2 = [self.F_isodata2, self.F_li2, self.F_mean2, self.F_otsu2, self.F_triangle2, self.F_yen2]

        if self.F_isodata2 == max(F_list2):
            tkinter.messagebox.showinfo(title="Find best (nucleus)", message="Choose isodata")
        elif self.F_li2 == max(F_list2):
            tkinter.messagebox.showinfo(title="Find best (nucleus)", message="Choose li")
        elif self.F_mean2 == max(F_list2):
            tkinter.messagebox.showinfo(title="Find best (nucleus)", message="Choose mean")
        elif self.F_otsu2 == max(F_list2):
            tkinter.messagebox.showinfo(title="Find best (nucleus)", message="Choose otsu")
        elif self.F_triangle2 == max(F_list2):
            tkinter.messagebox.showinfo(title="Find best (nucleus)", message="Choose triangle")
        else:
            tkinter.messagebox.showinfo(title="Find best (nucleus)", message="Choose yen")


    def showresult(self):
        if self.img_cyto is not None:
            self.result_cyto = Image.fromarray(self.img_cyto)
            self.img6 = customtkinter.CTkImage(Image.fromarray(self.img_cyto), size=(800, 800))
            self.sixth_frame_image_label = customtkinter.CTkLabel(self.sixth_frame, text="", image=self.img6)
            self.sixth_frame_image_label.grid(row=0, column=1, padx=25, pady=40)
        if self.img_nucl is not None:
            self.result_nucl = Image.fromarray(self.img_nucl)
            self.img7 = customtkinter.CTkImage(Image.fromarray(self.img_nucl), size=(800, 800))
            self.seventh_frame_image_label = customtkinter.CTkLabel(self.seventh_frame, text="", image=self.img7)
            self.seventh_frame_image_label.grid(row=0, column=1, padx=25, pady=40)

def run_gui():
    app = App()
    app.mainloop()

if __name__ == "__main__":
    run_gui()





import numpy as np
import cv2
from skimage.filters import thresholding
np.set_printoptions(threshold = np.inf)


def is_outlier(value, p25, p75):
    """Check if value is an outlier
    """
    lower = p25 - 1.5 * (p75 - p25)
    upper = p75 + 1.5 * (p75 - p25)
    return value <= lower or value >= upper


def get_indices_of_outliers(values):
    """Get outlier indices (if any)
    """
    p25 = np.percentile(values, 25)
    p75 = np.percentile(values, 75)

    indices_of_outliers = []
    for ind, value in enumerate(values):
        if is_outlier(value, p25, p75):
            indices_of_outliers.append(ind)
    upper = p75 + 1.5 * (p75 - p25)
    return indices_of_outliers, upper


def keep_largest_component(grid):
    rows = len(grid)
    cols = len(grid[0])
    visited = [[False] * cols for _ in range(rows)]
    largest_component = []

    for i in range(rows):
        for j in range(cols):
            if not visited[i][j] and grid[i][j] == 1:
                component = []
                dfs(grid, i, j, visited, component)
                if len(component) > len(largest_component):
                    largest_component = component

    largest_grid = [[0] * cols for _ in range(rows)]
    for i, j in largest_component:
        largest_grid[i][j] = 1

    return largest_grid


def dfs(grid, row, col, visited, component):
    rows = len(grid)
    cols = len(grid[0])
    stack = [(row, col)]

    while stack:
        curr_row, curr_col = stack.pop()
        if not visited[curr_row][curr_col] and grid[curr_row][curr_col] == 1:
            visited[curr_row][curr_col] = True
            component.append((curr_row, curr_col))

            # Explore neighbors in four directions (up, down, left, right)
            directions = [(-1, 0), (1, 0), (0, -1), (0, 1)]
            for dx, dy in directions:
                new_row = curr_row + dx
                new_col = curr_col + dy

                if 0 <= new_row < rows and 0 <= new_col < cols and not visited[new_row][new_col]:
                    stack.append((new_row, new_col))

def mostFrequent(arr, n):
    # Insert all elements in Hash.
    Hash = dict()
    for i in range(n):
        if arr[i] in Hash.keys():
            Hash[arr[i]] += 1
        else:
            Hash[arr[i]] = 1

    # find the max frequency
    max_count = 0
    res = -1
    for i in Hash:
        if max_count < Hash[i]:
            res = i
            max_count = Hash[i]
    return res


def isodata(file1, file2, k):

    img_ori = cv2.imread(file1, -1)
    segmented_cells = cv2.imread(file2, -1)

    rows = segmented_cells.shape[0]
    cols = segmented_cells.shape[1]

    s_c = segmented_cells.flatten()
    s_c = [x for x in s_c if x != 0]

    segmented_cells = np.uint8(segmented_cells)
    edge = cv2.Canny(segmented_cells, 0, max(s_c))

    img_contour = np.zeros((rows, cols))
    cell_num = np.amax(segmented_cells)
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour += edge

    img_blank1 = np.zeros((rows, cols))
    img_blank1[img_contour == 0] = 1
    img_blank2 = np.zeros((rows, cols))
    img_blank4 = np.zeros((rows, cols))
    img_blank5 = np.ones((rows, cols))

    img_blank2[segmented_cells > 0] = 1
    img_blank3 = np.multiply(img_ori, img_blank2)
    img_blank3 = np.multiply(img_blank3, img_blank1)
    img_blank6 = np.multiply(segmented_cells, img_blank1)

    threshold_list = img_blank3[img_blank3 != 0]
    indices_of_outliers, res = get_indices_of_outliers(threshold_list)
    if indices_of_outliers != []:
        thres1 = threshold_list[threshold_list > res][0]
    else:
        thres1 = res

    img_blank3[img_blank3 >= thres1] = 0
    thres2 = thresholding.threshold_isodata(img_blank3[img_blank3 != 0])

    img_blank5[img_blank3 < thres2] = 0
    img_blank5[img_blank3 == 0] = 1
    img_blank4 = img_blank2 - img_blank5

    img_global_nucl = np.multiply(img_blank4, segmented_cells)

    img_blank6 = np.uint8(img_blank6)

    # Perform distance transform on the grayscale image
    dist_transform = cv2.distanceTransform(img_blank6, cv2.DIST_L2, 3)
    dist_transform2 = np.zeros((rows, cols))
    dist_transform2[dist_transform > 0] = 1

    dist_transform2 = np.multiply(dist_transform2, segmented_cells)

    img_contour2 = np.zeros((rows, cols))

    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[dist_transform2 == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour2 += edge

    img_contour2[img_contour2 > 0] = 1
    boundary_pixel = np.multiply(img_contour2, dist_transform)
    s_c2 = boundary_pixel.flatten()
    s_c2 = [x for x in s_c2 if x != 0]
    average_num = sum(s_c2) / len(s_c2)

    # Determine the distance threshold as the valley between the peaks
    distance_threshold = average_num + k

    # Create a mask by setting pixels with distance values below the threshold to 255 (nucleus regions) and others to 0
    nucleus_mask = np.zeros_like(img_global_nucl)
    nucleus_mask[dist_transform < distance_threshold] = 255

    # Display the resulting nucleus mask

    img_blank7 = np.ones((rows, cols))
    img_blank7[nucleus_mask == 255] = 0
    nucleus_mask = np.multiply(img_blank7, img_global_nucl)
    img_blank8 = np.zeros((rows, cols))

    for i in range(max(s_c)):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1

        overlapping = np.multiply(nucleus_mask, img_blank)
        overlapping = np.uint8(overlapping)

        retval1, labels1, stats1, centroids1 = cv2.connectedComponentsWithStats(overlapping)
        img_labels = labels1.flatten()
        img_labels = [x for x in img_labels if x != 0]
        n = len(img_labels)
        mf_num = mostFrequent(img_labels, n)

        img_blank8[labels1 == mf_num] = 1

    img_blank9 = np.zeros((rows, cols))
    img_blank9[segmented_cells > 0] = 1
    img_blank9 -= img_blank8
    result = np.multiply(img_blank9, segmented_cells)

    final_result = np.zeros((rows, cols))
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[result == i + 1] = 1
        img_blank = np.uint8(img_blank)
        largest_grid = keep_largest_component(img_blank)
        final_result += largest_grid

    final_result_cyto = np.multiply(final_result, segmented_cells)
    final_result_nucl = segmented_cells - final_result_cyto
    return final_result_cyto, final_result_nucl

def li(file1, file2, k):

    img_ori = cv2.imread(file1, -1)
    segmented_cells = cv2.imread(file2, -1)

    rows = segmented_cells.shape[0]
    cols = segmented_cells.shape[1]

    s_c = segmented_cells.flatten()
    s_c = [x for x in s_c if x != 0]

    segmented_cells = np.uint8(segmented_cells)
    edge = cv2.Canny(segmented_cells, 0, max(s_c))

    img_contour = np.zeros((rows, cols))
    cell_num = np.amax(segmented_cells)
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour += edge

    img_blank1 = np.zeros((rows, cols))
    img_blank1[img_contour == 0] = 1
    img_blank2 = np.zeros((rows, cols))
    img_blank4 = np.zeros((rows, cols))
    img_blank5 = np.ones((rows, cols))

    img_blank2[segmented_cells > 0] = 1
    img_blank3 = np.multiply(img_ori, img_blank2)
    img_blank3 = np.multiply(img_blank3, img_blank1)
    img_blank6 = np.multiply(segmented_cells, img_blank1)

    threshold_list = img_blank3[img_blank3 != 0]
    indices_of_outliers, res = get_indices_of_outliers(threshold_list)
    if indices_of_outliers != []:
        thres1 = threshold_list[threshold_list > res][0]
    else:
        thres1 = res

    img_blank3[img_blank3 >= thres1] = 0
    thres2 = thresholding.threshold_li(img_blank3[img_blank3 != 0])

    img_blank5[img_blank3 < thres2] = 0
    img_blank5[img_blank3 == 0] = 1
    img_blank4 = img_blank2 - img_blank5

    img_global_nucl = np.multiply(img_blank4, segmented_cells)

    img_blank6 = np.uint8(img_blank6)

    # Perform distance transform on the grayscale image
    dist_transform = cv2.distanceTransform(img_blank6, cv2.DIST_L2, 3)
    dist_transform2 = np.zeros((rows, cols))
    dist_transform2[dist_transform > 0] = 1

    dist_transform2 = np.multiply(dist_transform2, segmented_cells)

    img_contour2 = np.zeros((rows, cols))

    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[dist_transform2 == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour2 += edge

    img_contour2[img_contour2 > 0] = 1
    boundary_pixel = np.multiply(img_contour2, dist_transform)
    s_c2 = boundary_pixel.flatten()
    s_c2 = [x for x in s_c2 if x != 0]
    average_num = sum(s_c2) / len(s_c2)

    # Determine the distance threshold as the valley between the peaks
    distance_threshold = average_num + k

    # Create a mask by setting pixels with distance values below the threshold to 255 (nucleus regions) and others to 0
    nucleus_mask = np.zeros_like(img_global_nucl)
    nucleus_mask[dist_transform < distance_threshold] = 255

    # Display the resulting nucleus mask

    img_blank7 = np.ones((rows, cols))
    img_blank7[nucleus_mask == 255] = 0
    nucleus_mask = np.multiply(img_blank7, img_global_nucl)
    img_blank8 = np.zeros((rows, cols))

    for i in range(max(s_c)):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1

        overlapping = np.multiply(nucleus_mask, img_blank)
        overlapping = np.uint8(overlapping)

        retval1, labels1, stats1, centroids1 = cv2.connectedComponentsWithStats(overlapping)
        img_labels = labels1.flatten()
        img_labels = [x for x in img_labels if x != 0]
        n = len(img_labels)
        mf_num = mostFrequent(img_labels, n)

        img_blank8[labels1 == mf_num] = 1

    img_blank9 = np.zeros((rows, cols))
    img_blank9[segmented_cells > 0] = 1
    img_blank9 -= img_blank8
    result = np.multiply(img_blank9, segmented_cells)

    final_result = np.zeros((rows, cols))
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[result == i + 1] = 1
        img_blank = np.uint8(img_blank)
        largest_grid = keep_largest_component(img_blank)
        final_result += largest_grid

    final_result_cyto = np.multiply(final_result, segmented_cells)
    final_result_nucl = segmented_cells - final_result_cyto
    return final_result_cyto, final_result_nucl

def mean(file1, file2, k):

    img_ori = cv2.imread(file1, -1)
    segmented_cells = cv2.imread(file2, -1)

    rows = segmented_cells.shape[0]
    cols = segmented_cells.shape[1]

    s_c = segmented_cells.flatten()
    s_c = [x for x in s_c if x != 0]

    segmented_cells = np.uint8(segmented_cells)
    edge = cv2.Canny(segmented_cells, 0, max(s_c))

    img_contour = np.zeros((rows, cols))
    cell_num = np.amax(segmented_cells)
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour += edge

    img_blank1 = np.zeros((rows, cols))
    img_blank1[img_contour == 0] = 1
    img_blank2 = np.zeros((rows, cols))
    img_blank4 = np.zeros((rows, cols))
    img_blank5 = np.ones((rows, cols))

    img_blank2[segmented_cells > 0] = 1
    img_blank3 = np.multiply(img_ori, img_blank2)
    img_blank3 = np.multiply(img_blank3, img_blank1)
    img_blank6 = np.multiply(segmented_cells, img_blank1)

    threshold_list = img_blank3[img_blank3 != 0]
    indices_of_outliers, res = get_indices_of_outliers(threshold_list)
    if indices_of_outliers != []:
        thres1 = threshold_list[threshold_list > res][0]
    else:
        thres1 = res

    img_blank3[img_blank3 >= thres1] = 0
    thres2 = thresholding.threshold_mean(img_blank3[img_blank3 != 0])

    img_blank5[img_blank3 < thres2] = 0
    img_blank5[img_blank3 == 0] = 1
    img_blank4 = img_blank2 - img_blank5

    img_global_nucl = np.multiply(img_blank4, segmented_cells)

    img_blank6 = np.uint8(img_blank6)

    # Perform distance transform on the grayscale image
    dist_transform = cv2.distanceTransform(img_blank6, cv2.DIST_L2, 3)
    dist_transform2 = np.zeros((rows, cols))
    dist_transform2[dist_transform > 0] = 1

    dist_transform2 = np.multiply(dist_transform2, segmented_cells)

    img_contour2 = np.zeros((rows, cols))

    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[dist_transform2 == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour2 += edge

    img_contour2[img_contour2 > 0] = 1
    boundary_pixel = np.multiply(img_contour2, dist_transform)
    s_c2 = boundary_pixel.flatten()
    s_c2 = [x for x in s_c2 if x != 0]
    average_num = sum(s_c2) / len(s_c2)

    # Determine the distance threshold as the valley between the peaks
    distance_threshold = average_num + k

    # Create a mask by setting pixels with distance values below the threshold to 255 (nucleus regions) and others to 0
    nucleus_mask = np.zeros_like(img_global_nucl)
    nucleus_mask[dist_transform < distance_threshold] = 255

    # Display the resulting nucleus mask

    img_blank7 = np.ones((rows, cols))
    img_blank7[nucleus_mask == 255] = 0
    nucleus_mask = np.multiply(img_blank7, img_global_nucl)
    img_blank8 = np.zeros((rows, cols))

    for i in range(max(s_c)):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1

        overlapping = np.multiply(nucleus_mask, img_blank)
        overlapping = np.uint8(overlapping)

        retval1, labels1, stats1, centroids1 = cv2.connectedComponentsWithStats(overlapping)
        img_labels = labels1.flatten()
        img_labels = [x for x in img_labels if x != 0]
        n = len(img_labels)
        mf_num = mostFrequent(img_labels, n)

        img_blank8[labels1 == mf_num] = 1

    img_blank9 = np.zeros((rows, cols))
    img_blank9[segmented_cells > 0] = 1
    img_blank9 -= img_blank8
    result = np.multiply(img_blank9, segmented_cells)

    final_result = np.zeros((rows, cols))
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[result == i + 1] = 1
        img_blank = np.uint8(img_blank)
        largest_grid = keep_largest_component(img_blank)
        final_result += largest_grid

    final_result_cyto = np.multiply(final_result, segmented_cells)
    final_result_nucl = segmented_cells - final_result_cyto
    return final_result_cyto, final_result_nucl

def otsu(file1, file2, k):

    img_ori = cv2.imread(file1, -1)
    segmented_cells = cv2.imread(file2, -1)

    rows = segmented_cells.shape[0]
    cols = segmented_cells.shape[1]

    s_c = segmented_cells.flatten()
    s_c = [x for x in s_c if x != 0]

    segmented_cells = np.uint8(segmented_cells)
    edge = cv2.Canny(segmented_cells, 0, max(s_c))

    img_contour = np.zeros((rows, cols))
    cell_num = np.amax(segmented_cells)
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour += edge

    img_blank1 = np.zeros((rows, cols))
    img_blank1[img_contour == 0] = 1
    img_blank2 = np.zeros((rows, cols))
    img_blank4 = np.zeros((rows, cols))
    img_blank5 = np.ones((rows, cols))

    img_blank2[segmented_cells > 0] = 1
    img_blank3 = np.multiply(img_ori, img_blank2)
    img_blank3 = np.multiply(img_blank3, img_blank1)
    img_blank6 = np.multiply(segmented_cells, img_blank1)

    threshold_list = img_blank3[img_blank3 != 0]
    indices_of_outliers, res = get_indices_of_outliers(threshold_list)
    if indices_of_outliers != []:
        thres1 = threshold_list[threshold_list > res][0]
    else:
        thres1 = res

    img_blank3[img_blank3 >= thres1] = 0
    thres2 = thresholding.threshold_otsu(img_blank3[img_blank3 != 0])

    img_blank5[img_blank3 < thres2] = 0
    img_blank5[img_blank3 == 0] = 1
    img_blank4 = img_blank2 - img_blank5

    img_global_nucl = np.multiply(img_blank4, segmented_cells)

    img_blank6 = np.uint8(img_blank6)

    # Perform distance transform on the grayscale image
    dist_transform = cv2.distanceTransform(img_blank6, cv2.DIST_L2, 3)
    dist_transform2 = np.zeros((rows, cols))
    dist_transform2[dist_transform > 0] = 1

    dist_transform2 = np.multiply(dist_transform2, segmented_cells)

    img_contour2 = np.zeros((rows, cols))

    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[dist_transform2 == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour2 += edge

    img_contour2[img_contour2 > 0] = 1
    boundary_pixel = np.multiply(img_contour2, dist_transform)
    s_c2 = boundary_pixel.flatten()
    s_c2 = [x for x in s_c2 if x != 0]
    average_num = sum(s_c2) / len(s_c2)

    # Determine the distance threshold as the valley between the peaks
    distance_threshold = average_num + k

    # Create a mask by setting pixels with distance values below the threshold to 255 (nucleus regions) and others to 0
    nucleus_mask = np.zeros_like(img_global_nucl)
    nucleus_mask[dist_transform < distance_threshold] = 255

    # Display the resulting nucleus mask

    img_blank7 = np.ones((rows, cols))
    img_blank7[nucleus_mask == 255] = 0
    nucleus_mask = np.multiply(img_blank7, img_global_nucl)
    img_blank8 = np.zeros((rows, cols))

    for i in range(max(s_c)):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1

        overlapping = np.multiply(nucleus_mask, img_blank)
        overlapping = np.uint8(overlapping)

        retval1, labels1, stats1, centroids1 = cv2.connectedComponentsWithStats(overlapping)
        img_labels = labels1.flatten()
        img_labels = [x for x in img_labels if x != 0]
        n = len(img_labels)
        mf_num = mostFrequent(img_labels, n)

        img_blank8[labels1 == mf_num] = 1

    img_blank9 = np.zeros((rows, cols))
    img_blank9[segmented_cells > 0] = 1
    img_blank9 -= img_blank8
    result = np.multiply(img_blank9, segmented_cells)

    final_result = np.zeros((rows, cols))
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[result == i + 1] = 1
        img_blank = np.uint8(img_blank)
        largest_grid = keep_largest_component(img_blank)
        final_result += largest_grid

    final_result_cyto = np.multiply(final_result, segmented_cells)
    final_result_nucl = segmented_cells - final_result_cyto
    return final_result_cyto, final_result_nucl

def triangle(file1, file2, k):

    img_ori = cv2.imread(file1, -1)
    segmented_cells = cv2.imread(file2, -1)

    rows = segmented_cells.shape[0]
    cols = segmented_cells.shape[1]

    s_c = segmented_cells.flatten()
    s_c = [x for x in s_c if x != 0]

    segmented_cells = np.uint8(segmented_cells)
    edge = cv2.Canny(segmented_cells, 0, max(s_c))

    img_contour = np.zeros((rows, cols))
    cell_num = np.amax(segmented_cells)
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour += edge

    img_blank1 = np.zeros((rows, cols))
    img_blank1[img_contour == 0] = 1
    img_blank2 = np.zeros((rows, cols))
    img_blank4 = np.zeros((rows, cols))
    img_blank5 = np.ones((rows, cols))

    img_blank2[segmented_cells > 0] = 1
    img_blank3 = np.multiply(img_ori, img_blank2)
    img_blank3 = np.multiply(img_blank3, img_blank1)
    img_blank6 = np.multiply(segmented_cells, img_blank1)

    threshold_list = img_blank3[img_blank3 != 0]
    indices_of_outliers, res = get_indices_of_outliers(threshold_list)
    if indices_of_outliers != []:
        thres1 = threshold_list[threshold_list > res][0]
    else:
        thres1 = res

    img_blank3[img_blank3 >= thres1] = 0
    thres2 = thresholding.threshold_triangle(img_blank3[img_blank3 != 0])

    img_blank5[img_blank3 < thres2] = 0
    img_blank5[img_blank3 == 0] = 1
    img_blank4 = img_blank2 - img_blank5

    img_global_nucl = np.multiply(img_blank4, segmented_cells)

    img_blank6 = np.uint8(img_blank6)

    # Perform distance transform on the grayscale image
    dist_transform = cv2.distanceTransform(img_blank6, cv2.DIST_L2, 3)
    dist_transform2 = np.zeros((rows, cols))
    dist_transform2[dist_transform > 0] = 1

    dist_transform2 = np.multiply(dist_transform2, segmented_cells)

    img_contour2 = np.zeros((rows, cols))

    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[dist_transform2 == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour2 += edge

    img_contour2[img_contour2 > 0] = 1
    boundary_pixel = np.multiply(img_contour2, dist_transform)
    s_c2 = boundary_pixel.flatten()
    s_c2 = [x for x in s_c2 if x != 0]
    average_num = sum(s_c2) / len(s_c2)

    # Determine the distance threshold as the valley between the peaks
    distance_threshold = average_num + k

    # Create a mask by setting pixels with distance values below the threshold to 255 (nucleus regions) and others to 0
    nucleus_mask = np.zeros_like(img_global_nucl)
    nucleus_mask[dist_transform < distance_threshold] = 255

    # Display the resulting nucleus mask

    img_blank7 = np.ones((rows, cols))
    img_blank7[nucleus_mask == 255] = 0
    nucleus_mask = np.multiply(img_blank7, img_global_nucl)
    img_blank8 = np.zeros((rows, cols))

    for i in range(max(s_c)):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1

        overlapping = np.multiply(nucleus_mask, img_blank)
        overlapping = np.uint8(overlapping)

        retval1, labels1, stats1, centroids1 = cv2.connectedComponentsWithStats(overlapping)
        img_labels = labels1.flatten()
        img_labels = [x for x in img_labels if x != 0]
        n = len(img_labels)
        mf_num = mostFrequent(img_labels, n)

        img_blank8[labels1 == mf_num] = 1

    img_blank9 = np.zeros((rows, cols))
    img_blank9[segmented_cells > 0] = 1
    img_blank9 -= img_blank8
    result = np.multiply(img_blank9, segmented_cells)

    final_result = np.zeros((rows, cols))
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[result == i + 1] = 1
        img_blank = np.uint8(img_blank)
        largest_grid = keep_largest_component(img_blank)
        final_result += largest_grid

    final_result_cyto = np.multiply(final_result, segmented_cells)
    final_result_nucl = segmented_cells - final_result_cyto
    return final_result_cyto, final_result_nucl

def yen(file1, file2, k):

    img_ori = cv2.imread(file1, -1)
    segmented_cells = cv2.imread(file2, -1)

    rows = segmented_cells.shape[0]
    cols = segmented_cells.shape[1]

    s_c = segmented_cells.flatten()
    s_c = [x for x in s_c if x != 0]

    segmented_cells = np.uint8(segmented_cells)
    edge = cv2.Canny(segmented_cells, 0, max(s_c))

    img_contour = np.zeros((rows, cols))
    cell_num = np.amax(segmented_cells)
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour += edge

    img_blank1 = np.zeros((rows, cols))
    img_blank1[img_contour == 0] = 1
    img_blank2 = np.zeros((rows, cols))
    img_blank4 = np.zeros((rows, cols))
    img_blank5 = np.ones((rows, cols))

    img_blank2[segmented_cells > 0] = 1
    img_blank3 = np.multiply(img_ori, img_blank2)
    img_blank3 = np.multiply(img_blank3, img_blank1)
    img_blank6 = np.multiply(segmented_cells, img_blank1)

    threshold_list = img_blank3[img_blank3 != 0]
    indices_of_outliers, res = get_indices_of_outliers(threshold_list)
    if indices_of_outliers != []:
        thres1 = threshold_list[threshold_list > res][0]
    else:
        thres1 = res

    img_blank3[img_blank3 >= thres1] = 0
    thres2 = thresholding.threshold_yen(img_blank3[img_blank3 != 0])

    img_blank5[img_blank3 < thres2] = 0
    img_blank5[img_blank3 == 0] = 1
    img_blank4 = img_blank2 - img_blank5

    img_global_nucl = np.multiply(img_blank4, segmented_cells)

    img_blank6 = np.uint8(img_blank6)

    # Perform distance transform on the grayscale image
    dist_transform = cv2.distanceTransform(img_blank6, cv2.DIST_L2, 3)
    dist_transform2 = np.zeros((rows, cols))
    dist_transform2[dist_transform > 0] = 1

    dist_transform2 = np.multiply(dist_transform2, segmented_cells)

    img_contour2 = np.zeros((rows, cols))

    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[dist_transform2 == i + 1] = 1
        img_blank = np.uint8(img_blank)

        edge = cv2.Canny(img_blank, 0, 1)
        img_contour2 += edge

    img_contour2[img_contour2 > 0] = 1
    boundary_pixel = np.multiply(img_contour2, dist_transform)
    s_c2 = boundary_pixel.flatten()
    s_c2 = [x for x in s_c2 if x != 0]
    average_num = sum(s_c2) / len(s_c2)

    # Determine the distance threshold as the valley between the peaks
    distance_threshold = average_num + k

    # Create a mask by setting pixels with distance values below the threshold to 255 (nucleus regions) and others to 0
    nucleus_mask = np.zeros_like(img_global_nucl)
    nucleus_mask[dist_transform < distance_threshold] = 255

    # Display the resulting nucleus mask

    img_blank7 = np.ones((rows, cols))
    img_blank7[nucleus_mask == 255] = 0
    nucleus_mask = np.multiply(img_blank7, img_global_nucl)
    img_blank8 = np.zeros((rows, cols))

    for i in range(max(s_c)):
        img_blank = np.zeros((rows, cols))
        img_blank[segmented_cells == i + 1] = 1

        overlapping = np.multiply(nucleus_mask, img_blank)
        overlapping = np.uint8(overlapping)

        retval1, labels1, stats1, centroids1 = cv2.connectedComponentsWithStats(overlapping)
        img_labels = labels1.flatten()
        img_labels = [x for x in img_labels if x != 0]
        n = len(img_labels)
        mf_num = mostFrequent(img_labels, n)

        img_blank8[labels1 == mf_num] = 1

    img_blank9 = np.zeros((rows, cols))
    img_blank9[segmented_cells > 0] = 1
    img_blank9 -= img_blank8
    result = np.multiply(img_blank9, segmented_cells)

    final_result = np.zeros((rows, cols))
    for i in range(cell_num):
        img_blank = np.zeros((rows, cols))
        img_blank[result == i + 1] = 1
        img_blank = np.uint8(img_blank)
        largest_grid = keep_largest_component(img_blank)
        final_result += largest_grid

    final_result_cyto = np.multiply(final_result, segmented_cells)
    final_result_nucl = segmented_cells - final_result_cyto
    return final_result_cyto, final_result_nucl





from distutils.core import setup
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
  name='Cellcyto',
  packages=['Cellcyto'],
  version='1.8.1',
  license='Walsh Lab',
  description='This algorithm is designed for cell segmentation post processing to find nucleus and cytoplasm, and evaluation of the segmentation algorithm based on ground truth images',
  author='Nianchao Wang',
  author_email='nwang27@tamu.edu',
  keywords=['Cell Segmentation', 'POSEA', 'Cytoplasmic Post-processing Algorithm (CPPA)'],
  long_description=long_description,
  long_description_content_type="text/markdown",
  install_requires=[
      'customtkinter',
      'opencv-python',
      'numpy',
      'pillow',
      'pandas',
      'scikit-image',
  ],
  classifiers=[
      'Development Status :: 3 - Alpha',
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: BSD License',
      'Programming Language :: Python :: 3',
  ],
  entry_points={
      'console_scripts': [
          'Cellcyto=Cellcyto.GUI:run_gui',
      ],
  },
)



import cv2
import numpy as np
from PIL import Image
np.set_printoptions(threshold = np.inf)
import math as mt
import pandas as pd


def POSEA(file1, file2):
    def mostFrequent(arr, n):
        # Insert all elements in Hash.
        Hash = dict()
        for i in range(n):
            if arr[i] in Hash.keys():
                Hash[arr[i]] += 1
            else:
                Hash[arr[i]] = 1

        # find the max frequency
        max_count = 0
        res = -1
        for i in Hash:
            if (max_count < Hash[i]):
                res = i
                max_count = Hash[i]
        return res

    img_cp = cv2.imread(file1, -1)
    img_gt = cv2.imread(file2, -1)
    img_cp = img_cp.astype(int)

    img_gt = img_gt.astype(int)
    img_gtt = img_gt.flatten()

    img_gtt = [x for x in img_gtt if x != 0]
    n = len(img_gtt)
    level1 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level1]
    n = len(img_gtt)
    level2 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level2]
    n = len(img_gtt)
    level3 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level3]
    n = len(img_gtt)
    level4 = mostFrequent(img_gtt, n)

    img_gt1 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt1[img_gt == level1] = 1
    img_gt2 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt2[img_gt == level2] = 1
    img_gt3 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt3[img_gt == level3] = 1
    img_gt4 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt4[img_gt == level4] = 1
    img_gt_mask = img_gt1 + img_gt2 + img_gt3 + img_gt4

    img_cp_mask = np.zeros((img_cp.shape[0], img_cp.shape[1]))
    img_cp_mask[img_cp > 0] = 1
    area_cp = sum(sum(img_cp_mask))
    combine = img_cp_mask + img_gt_mask
    combine[combine == 1] = 0
    combine[combine == 2] = 1
    TP = sum(sum(combine))
    FP = area_cp - TP
    FN = sum(sum(img_gt_mask)) - TP

    Pre_mask_cp = TP / (TP + FP)

    Re_mask_cp = TP / (TP + FN)

    F_mask_cp = 2 * Pre_mask_cp * Re_mask_cp / (Pre_mask_cp + Re_mask_cp)

    # ######Object part

    img_gt1 = np.array(img_gt1, dtype=np.uint8)
    img_gt2 = np.array(img_gt2, dtype=np.uint8)
    img_gt3 = np.array(img_gt3, dtype=np.uint8)
    img_gt4 = np.array(img_gt4, dtype=np.uint8)

    _, labels1, stats1, _ = cv2.connectedComponentsWithStats(img_gt1)
    _, labels2, stats2, _ = cv2.connectedComponentsWithStats(img_gt2)
    _, labels3, stats3, _ = cv2.connectedComponentsWithStats(img_gt3)
    _, labels4, stats4, _ = cv2.connectedComponentsWithStats(img_gt4)

    num_gt1, _ = np.shape(stats1)
    num_gt1 -= 1
    num_gt2, _ = np.shape(stats2)
    num_gt2 -= 1
    num_gt3, _ = np.shape(stats3)
    num_gt3 -= 1
    num_gt4, _ = np.shape(stats4)
    num_gt4 -= 1
    num_total = num_gt1 + num_gt2 + num_gt3 + num_gt4

    for i in range(num_gt2):
        labels2[labels2 == i + 1] = num_gt1 + 1 + i

    for i in range(num_gt3):
        labels3[labels3 == i + 1] = num_gt1 + num_gt2 + 1 + i

    for i in range(num_gt4):
        labels4[labels4 == i + 1] = num_gt1 + num_gt2 + num_gt3 + 1 + i

    img_relabel = labels1 + labels2 + labels3 + labels4

    cell_each = []
    cell_each_F = []
    cell_each_P = []
    cell_each_R = []

    TP_cp_obj = 0
    FP_cp_obj = 0
    cp_match_mask = 0
    fn_cp_obj = 0

    for i in range(num_total):
        img_blank = np.zeros((img_gt.shape[0], img_gt.shape[1]))
        img_blank[img_relabel == i + 1] = 1
        GT_area = sum(sum(img_blank))
        overlapping = np.multiply(img_cp, img_blank)

        if sum(sum(overlapping)) == 0:
            fn_cp_obj += sum(sum(img_blank))
            cell_each.append(0)

        overlapping_cp_obj = overlapping.flatten()
        overlapping_cp_obj = overlapping_cp_obj[overlapping_cp_obj != 0]
        overlapping_cp_obj = overlapping_cp_obj.astype(int)
        counts_cp_obj = np.bincount(overlapping_cp_obj)

        if counts_cp_obj.size > 0:
            mf_value_cp_obj = np.argmax(counts_cp_obj)
            img_blank = np.zeros((img_cp.shape[0], img_cp.shape[1]))
            img_blank[overlapping == mf_value_cp_obj] = 1
            TP_cp_obj += sum(sum(img_blank))

            img_ones = np.ones((img_cp.shape[0], img_cp.shape[1]))
            img_ones2 = np.ones((img_cp.shape[0], img_cp.shape[1]))
            img_ones[img_blank > 0] = 0

            difference_cp_1 = (sum(sum(np.multiply(img_ones2, img_cp))) - sum(
                sum(np.multiply(img_ones, img_cp)))) / sum(sum(img_blank))
            img_blank_cp = np.zeros((img_cp.shape[0], img_cp.shape[1]))
            img_blank_cp[img_cp == difference_cp_1] = 1
            cp_match_mask += img_blank_cp
            FP_cp_obj += (sum(sum(img_blank_cp)) - sum(sum(img_blank)))

            TP_cell1 = sum(sum(img_blank))
            FP_cell1 = sum(sum(img_blank_cp)) - sum(sum(img_blank))
            FN_cell1 = GT_area - sum(sum(img_blank))

            Pre_cell1 = TP_cell1 / (TP_cell1 + FP_cell1)
            Re_cell1 = TP_cell1 / (TP_cell1 + FN_cell1)
            F_cell1 = 2 * Pre_cell1 * Re_cell1 / (Pre_cell1 + Re_cell1)
            cell_each_P.append(Pre_cell1)
            cell_each_R.append(Re_cell1)
            cell_each_F.append(F_cell1)

    img_blank = np.zeros((img_cp.shape[0], img_cp.shape[1]))
    img_blank[img_cp > 0] = 1
    img_cp_mask = img_blank

    cp_match_mask[cp_match_mask > 0] = 1

    FP_cp_obj = FP_cp_obj + sum(sum(img_cp_mask)) - sum(sum(cp_match_mask))
    FN_cp_obj = sum(sum(img_gt_mask)) - TP_cp_obj

    F_cp_obj = (2 * TP_cp_obj) / (2 * TP_cp_obj + FN_cp_obj + FP_cp_obj)

    Pre_cp_obj = TP_cp_obj / (TP_cp_obj + FP_cp_obj)

    Re_cp_obj = TP_cp_obj / (TP_cp_obj + FN_cp_obj)

    cell_each_F.extend(cell_each)
    cell_each_R.extend(cell_each)
    cell_each_P.extend(cell_each)

    cell_each_F = np.array(cell_each_F)
    cell_each_R = np.array(cell_each_R)
    cell_each_P = np.array(cell_each_P)

    cell_each_F_average = sum(cell_each_F) / len(cell_each_F)
    cell_each_R_average = sum(cell_each_R) / len(cell_each_R)
    cell_each_P_average = sum(cell_each_P) / len(cell_each_P)

    cell_each_F = pd.DataFrame([[i] for i in np.array(cell_each_F)])
    cell_each_R = pd.DataFrame([[i] for i in np.array(cell_each_R)])
    cell_each_P = pd.DataFrame([[i] for i in np.array(cell_each_P)])

    data = np.column_stack([cell_each_F, cell_each_P, cell_each_R])

    data = pd.DataFrame(data=data, columns=['F-measure', 'Precision', 'Recall'])

    return F_mask_cp, Pre_mask_cp, Re_mask_cp, F_cp_obj, Pre_cp_obj, Re_cp_obj, cell_each_F_average, cell_each_P_average, cell_each_R_average, data

def POSEA2(file1, file2, file3):

    def mostFrequent(arr, n):
        # Insert all elements in Hash.
        Hash = dict()
        for i in range(n):
            if arr[i] in Hash.keys():
                Hash[arr[i]] += 1
            else:
                Hash[arr[i]] = 1

        # find the max frequency
        max_count = 0
        res = -1
        for i in Hash:
            if (max_count < Hash[i]):
                res = i
                max_count = Hash[i]
        return res


    img_cp = np.array(file1)
    img_gt = cv2.imread(file2, -1)
    img_gt_CorNmask = cv2.imread(file3, -1)

    img_cp = img_cp.astype(int)
    img_gt = img_gt.astype(int)
    img_gt_CorNmask = img_gt_CorNmask.astype(int)
    img_gtt = img_gt.flatten()


    img_gtt = [x for x in img_gtt if x != 0]
    n = len(img_gtt)
    level1 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level1]
    n = len(img_gtt)
    level2 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level2]
    n = len(img_gtt)
    level3 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level3]
    n = len(img_gtt)
    level4 = mostFrequent(img_gtt, n)

    img_gt1 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt1[img_gt == level1] = 1
    img_gt2 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt2[img_gt == level2] = 1
    img_gt3 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt3[img_gt == level3] = 1
    img_gt4 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt4[img_gt == level4] = 1
    img_gt_mask = img_gt1 + img_gt2 + img_gt3 + img_gt4

    img_gt_CorNmask[img_gt_CorNmask > 0] = 1
    img_cp_mask = np.zeros((img_cp.shape[0], img_cp.shape[1]))
    img_cp_mask[img_cp > 0] = 1
    area_cp = sum(sum(img_cp_mask))
    combine = img_cp_mask + img_gt_CorNmask
    combine[combine == 1] = 0
    combine[combine == 2] = 1
    TP = sum(sum(combine))
    FP = area_cp - TP
    FN = sum(sum(img_gt_CorNmask)) - TP

    Pre_mask_cp = TP / (TP + FP)

    Re_mask_cp = TP / (TP + FN)

    F_mask_cp = 2 * Pre_mask_cp * Re_mask_cp / (Pre_mask_cp + Re_mask_cp)

    # ######Object part

    img_gt1 = np.array(img_gt1, dtype=np.uint8)
    img_gt2 = np.array(img_gt2, dtype=np.uint8)
    img_gt3 = np.array(img_gt3, dtype=np.uint8)
    img_gt4 = np.array(img_gt4, dtype=np.uint8)

    _, labels1, stats1, _ = cv2.connectedComponentsWithStats(img_gt1)
    _, labels2, stats2, _ = cv2.connectedComponentsWithStats(img_gt2)
    _, labels3, stats3, _ = cv2.connectedComponentsWithStats(img_gt3)
    _, labels4, stats4, _ = cv2.connectedComponentsWithStats(img_gt4)

    num_gt1, _ = np.shape(stats1)
    num_gt1 -= 1
    num_gt2, _ = np.shape(stats2)
    num_gt2 -= 1
    num_gt3, _ = np.shape(stats3)
    num_gt3 -= 1
    num_gt4, _ = np.shape(stats4)
    num_gt4 -= 1
    num_total = num_gt1 + num_gt2 + num_gt3 + num_gt4

    for i in range(num_gt2):
        labels2[labels2 == i + 1] = num_gt1 + 1 + i

    for i in range(num_gt3):
        labels3[labels3 == i + 1] = num_gt1 + num_gt2 + 1 + i

    for i in range(num_gt4):
        labels4[labels4 == i + 1] = num_gt1 + num_gt2 + num_gt3 + 1 + i

    img_relabel = labels1 + labels2 + labels3 + labels4
    img_relabel = np.multiply(img_gt_CorNmask, img_relabel)

    cell_each = []
    cell_each_F = []
    cell_each_P = []
    cell_each_R = []

    TP_cp_obj = 0
    FP_cp_obj = 0
    cp_match_mask = 0
    fn_cp_obj = 0

    for i in range(num_total):
        img_blank = np.zeros((img_gt.shape[0], img_gt.shape[1]))
        img_blank[img_relabel == i + 1] = 1
        GT_area = sum(sum(img_blank))
        overlapping = np.multiply(img_cp, img_blank)

        if sum(sum(overlapping)) == 0:
            fn_cp_obj += sum(sum(img_blank))
            cell_each.append(0)

        overlapping_cp_obj = overlapping.flatten()
        overlapping_cp_obj = overlapping_cp_obj[overlapping_cp_obj != 0]
        overlapping_cp_obj = overlapping_cp_obj.astype(int)
        counts_cp_obj = np.bincount(overlapping_cp_obj)

        if counts_cp_obj.size > 0:
            mf_value_cp_obj = np.argmax(counts_cp_obj)
            img_blank = np.zeros((img_cp.shape[0], img_cp.shape[1]))
            img_blank[overlapping == mf_value_cp_obj] = 1
            TP_cp_obj += sum(sum(img_blank))

            img_ones = np.ones((img_cp.shape[0], img_cp.shape[1]))
            img_ones2 = np.ones((img_cp.shape[0], img_cp.shape[1]))
            img_ones[img_blank > 0] = 0

            difference_cp_1 = (sum(sum(np.multiply(img_ones2, img_cp))) - sum(
                sum(np.multiply(img_ones, img_cp)))) / sum(sum(img_blank))
            img_blank_cp = np.zeros((img_cp.shape[0], img_cp.shape[1]))
            img_blank_cp[img_cp == difference_cp_1] = 1
            cp_match_mask += img_blank_cp
            FP_cp_obj += (sum(sum(img_blank_cp)) - sum(sum(img_blank)))

            TP_cell1 = sum(sum(img_blank))
            FP_cell1 = sum(sum(img_blank_cp)) - sum(sum(img_blank))
            FN_cell1 = GT_area - sum(sum(img_blank))

            Pre_cell1 = TP_cell1 / (TP_cell1 + FP_cell1)
            Re_cell1 = TP_cell1 / (TP_cell1 + FN_cell1)
            F_cell1 = 2 * Pre_cell1 * Re_cell1 / (Pre_cell1 + Re_cell1)
            cell_each_P.append(Pre_cell1)
            cell_each_R.append(Re_cell1)
            cell_each_F.append(F_cell1)

    img_blank = np.zeros((img_cp.shape[0], img_cp.shape[1]))
    img_blank[img_cp > 0] = 1
    img_cp_mask = img_blank

    cp_match_mask[cp_match_mask > 0] = 1

    FP_cp_obj = FP_cp_obj + sum(sum(img_cp_mask)) - sum(sum(cp_match_mask))
    FN_cp_obj = sum(sum(img_gt_CorNmask)) - TP_cp_obj

    F_cp_obj = (2 * TP_cp_obj) / (2 * TP_cp_obj + FN_cp_obj + FP_cp_obj)

    Pre_cp_obj = TP_cp_obj / (TP_cp_obj + FP_cp_obj)

    Re_cp_obj = TP_cp_obj / (TP_cp_obj + FN_cp_obj)

    cell_each_F.extend(cell_each)
    cell_each_R.extend(cell_each)
    cell_each_P.extend(cell_each)

    cell_each_F = np.array(cell_each_F)
    cell_each_R = np.array(cell_each_R)
    cell_each_P = np.array(cell_each_P)

    cell_each_F_average = sum(cell_each_F) / len(cell_each_F)
    cell_each_R_average = sum(cell_each_R) / len(cell_each_R)
    cell_each_P_average = sum(cell_each_P) / len(cell_each_P)

    cell_each_F = pd.DataFrame([[i] for i in np.array(cell_each_F)])
    cell_each_R = pd.DataFrame([[i] for i in np.array(cell_each_R)])
    cell_each_P = pd.DataFrame([[i] for i in np.array(cell_each_P)])

    data = np.column_stack([cell_each_F, cell_each_P, cell_each_R])

    data = pd.DataFrame(data=data, columns=['F-measure', 'Precision', 'Recall'])

    return F_mask_cp, Pre_mask_cp, Re_mask_cp, F_cp_obj, Pre_cp_obj, Re_cp_obj, cell_each_F_average, cell_each_P_average, cell_each_R_average, data



def POSEA3(file1, file2, file3):

    def mostFrequent(arr, n):
        # Insert all elements in Hash.
        Hash = dict()
        for i in range(n):
            if arr[i] in Hash.keys():
                Hash[arr[i]] += 1
            else:
                Hash[arr[i]] = 1

        # find the max frequency
        max_count = 0
        res = -1
        for i in Hash:
            if (max_count < Hash[i]):
                res = i
                max_count = Hash[i]
        return res


    img_cp = np.array(file1)
    img_gt = cv2.imread(file2, -1)
    img_gt_CorNmask = cv2.imread(file3, -1)

    img_cp = img_cp.astype(int)
    img_gt = img_gt.astype(int)
    img_gt_CorNmask = img_gt_CorNmask.astype(int)
    img_gtt = img_gt.flatten()


    img_gtt = [x for x in img_gtt if x != 0]
    n = len(img_gtt)
    level1 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level1]
    n = len(img_gtt)
    level2 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level2]
    n = len(img_gtt)
    level3 = mostFrequent(img_gtt, n)

    img_gtt = [x for x in img_gtt if x != level3]
    n = len(img_gtt)
    level4 = mostFrequent(img_gtt, n)

    img_gt1 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt1[img_gt == level1] = 1
    img_gt2 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt2[img_gt == level2] = 1
    img_gt3 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt3[img_gt == level3] = 1
    img_gt4 = np.zeros((img_gt.shape[0], img_gt.shape[1]))
    img_gt4[img_gt == level4] = 1
    img_gt_mask = img_gt1 + img_gt2 + img_gt3 + img_gt4
    img_gt_CorNmask[img_gt_CorNmask > 0] = 1


    # ######Object part

    img_gt1 = np.array(img_gt1, dtype=np.uint8)
    img_gt2 = np.array(img_gt2, dtype=np.uint8)
    img_gt3 = np.array(img_gt3, dtype=np.uint8)
    img_gt4 = np.array(img_gt4, dtype=np.uint8)

    _, labels1, stats1, _ = cv2.connectedComponentsWithStats(img_gt1)
    _, labels2, stats2, _ = cv2.connectedComponentsWithStats(img_gt2)
    _, labels3, stats3, _ = cv2.connectedComponentsWithStats(img_gt3)
    _, labels4, stats4, _ = cv2.connectedComponentsWithStats(img_gt4)

    num_gt1, _ = np.shape(stats1)
    num_gt1 -= 1
    num_gt2, _ = np.shape(stats2)
    num_gt2 -= 1
    num_gt3, _ = np.shape(stats3)
    num_gt3 -= 1
    num_gt4, _ = np.shape(stats4)
    num_gt4 -= 1
    num_total = num_gt1 + num_gt2 + num_gt3 + num_gt4

    for i in range(num_gt2):
        labels2[labels2 == i + 1] = num_gt1 + 1 + i

    for i in range(num_gt3):
        labels3[labels3 == i + 1] = num_gt1 + num_gt2 + 1 + i

    for i in range(num_gt4):
        labels4[labels4 == i + 1] = num_gt1 + num_gt2 + num_gt3 + 1 + i

    img_relabel = labels1 + labels2 + labels3 + labels4
    img_relabel = np.multiply(img_gt_CorNmask, img_relabel)

    cell_each = []
    cell_each_F = []
    cell_each_P = []
    cell_each_R = []

    TP_cp_obj = 0
    FP_cp_obj = 0
    cp_match_mask = 0
    fn_cp_obj = 0

    for i in range(num_total):
        img_blank = np.zeros((img_gt.shape[0], img_gt.shape[1]))
        img_blank[img_relabel == i + 1] = 1
        GT_area = sum(sum(img_blank))
        overlapping = np.multiply(img_cp, img_blank)

        if sum(sum(overlapping)) == 0:
            fn_cp_obj += sum(sum(img_blank))
            cell_each.append(0)

        overlapping_cp_obj = overlapping.flatten()
        overlapping_cp_obj = overlapping_cp_obj[overlapping_cp_obj != 0]
        overlapping_cp_obj = overlapping_cp_obj.astype(int)
        counts_cp_obj = np.bincount(overlapping_cp_obj)

        if counts_cp_obj.size > 0:
            mf_value_cp_obj = np.argmax(counts_cp_obj)
            img_blank = np.zeros((img_cp.shape[0], img_cp.shape[1]))
            img_blank[overlapping == mf_value_cp_obj] = 1
            TP_cp_obj += sum(sum(img_blank))

            img_ones = np.ones((img_cp.shape[0], img_cp.shape[1]))
            img_ones2 = np.ones((img_cp.shape[0], img_cp.shape[1]))
            img_ones[img_blank > 0] = 0

            difference_cp_1 = (sum(sum(np.multiply(img_ones2, img_cp))) - sum(
                sum(np.multiply(img_ones, img_cp)))) / sum(sum(img_blank))
            img_blank_cp = np.zeros((img_cp.shape[0], img_cp.shape[1]))
            img_blank_cp[img_cp == difference_cp_1] = 1
            cp_match_mask += img_blank_cp
            FP_cp_obj += (sum(sum(img_blank_cp)) - sum(sum(img_blank)))


    img_blank = np.zeros((img_cp.shape[0], img_cp.shape[1]))
    img_blank[img_cp > 0] = 1
    img_cp_mask = img_blank

    cp_match_mask[cp_match_mask > 0] = 1

    FP_cp_obj = FP_cp_obj + sum(sum(img_cp_mask)) - sum(sum(cp_match_mask))
    FN_cp_obj = sum(sum(img_gt_CorNmask)) - TP_cp_obj

    F_cp_obj = (2 * TP_cp_obj) / (2 * TP_cp_obj + FN_cp_obj + FP_cp_obj)


    return F_cp_obj




